'use strict';
const zonaNamespace = 'org.hotelandia.zona';
const zonaType = 'Zona';

// 1. Connect
const bnUtil = require('./businessnetwork-connection-util');
bnUtil.connect(main);

function main(error){
    // Check for the connection error
    if(error){
        console.log(error);
        process.exit(1);
    }
    // 2. Get the zona AssetRegistry
    return bnUtil.connection.getAssetRegistry(zonaNamespace+'.'+zonaType).then((registry)=>{
        console.log('1. Received Registry: ', registry.id);

        // Utility method for adding the zonas
        addZonas(registry);

    }).catch((error)=>{
        console.log(error);
       // bnUtil.disconnect();
    });
}
/**
 * Creates two resources instances 
 * @param {*} registry This is of type AssetRegistry
 */
function    addZonas(registry){
    // 3. This Array will hold the instances of zonas resource
    let    zonas = [];
    const  bnDef = bnUtil.connection.getBusinessNetwork();
    const  factory = bnDef.getFactory();
    // Instance#1
    let    zonaResource = factory.newResource(zonaNamespace,zonaType,'BAR');
    zonaResource.setPropertyValue('ownershipType','LEASED');
    zonaResource.setPropertyValue('suites',10);
    zonaResource.setPropertyValue('dobles',10);
    zonaResource.setPropertyValue('individuales',50);
    // Push instance to  the zonas array
    zonas.push(zonaResource);

    // Instance#2 
    zonaResource = factory.newResource(zonaNamespace,zonaType,'BAR');
    // You may use direct assignment instead of using the setPropertyValue()
    zonaResource.ownershipType='OWNED';
    zonaResource.suites=15
    zonaResource.dobles=20;
    zonaResource.individuales=100;
    // Push instance to  the zonas array
    zonas.push(zonaResource);

    // 4. Add the zona resource to the registry
    return registry.addAll(zonas).then(()=>{
        console.log('Added the Resources successfully!!!');
        bnUtil.disconnect();
    }).catch((error)=>{
        console.log(error);
        bnUtil.disconnect();
    });
}
