'use strict';

 // Constant values 
 const namespace = "org.hotelandia.hotel";
 const transactionType = "CreateHotel";

// 1. Connect to hotelandia
const bnUtil = require('./businessnetwork-connection-util');
bnUtil.connect(main);

function main(error){
    
    // Check for error
    if(error){
        console.log(error);
        process.exit(1);
    }

    // 2. Get the Business Network Definition
    let bnDef =  bnUtil.connection.getBusinessNetwork();
    console.log("2. Received Definition from Runtime: ",
                   bnDef.getName(),"  ",bnDef.getVersion());

    // 3. Get the factory
    let factory = bnDef.getFactory();
    
    // 4. Lets create a new Resource of type = Transaction
    //    Here is the sample data
    // {
    //     "$class": "org.hotelandia.hotel.CreateHotel",
    //     "HotelNumber": "ZAR01-06-06-2019",
	 //    "code": "ZAR",
    //     "calle": "C/alegria",
    //     "schedule": "2019-06-06T18:49:58.273Z"
    // }

    // 4. Create an instance of transaction
    let options = {
        generate: false,
        includeOptionalFields: false
    }
    let HotelId = "ZAR01-01-10-19";
    let transaction = factory.newTransaction(namespace,transactionType,HotelId,options);

    // 5. Set up the properties of the transaction object
    transaction.setPropertyValue('hotelNumber', 'ZAR03');
    transaction.setPropertyValue('code', 'ZAR');
    transaction.setPropertyValue('street' , 'calle tercera');
    transaction.setPropertyValue('schedule' , new Date('2019-10-01T18:18:12.432Z'));

    // 6. Submit the transaction
    return bnUtil.connection.submitTransaction(transaction).then(()=>{
        console.log("6. Transaction Submitted/Processed Successfully!!")

        bnUtil.disconnect();

    }).catch((error)=>{
        console.log(error);

        bnUtil.disconnect();
    });
}




